#include "apitwitter.h"

/*
 * ################################################################################################
 * #                                         TokenFetcher                                         #
 * ################################################################################################
 */

TokenFetcher::TokenFetcher(QWidget *parent) : QWidget(parent)
{
	oauthInterface = new QOAuth::Interface(this);
	manager = new QNetworkAccessManager(this);
}

TokenFetcher::~TokenFetcher()
{
	delete oauthInterface;
	delete manager;
}

void TokenFetcher::getAccess()
{
	oauthInterface->setConsumerKey(CONSUMER_KEY);
	oauthInterface->setConsumerSecret(CONSUMER_SECRET_KEY);
	oauthInterface->setRequestTimeout(10000);

	QOAuth::ParamMap args;
		args.insert("oauth_callback", "oob");
	QOAuth::ParamMap reply = oauthInterface->requestToken("https://api.twitter.com/oauth/request_token", QOAuth::POST, QOAuth::HMAC_SHA1, args);

	if(oauthInterface->error() == QOAuth::NoError)
	{
		token = reply.value(QOAuth::tokenParameterName());
		tokenSecret = reply.value(QOAuth::tokenSecretParameterName());

		QDesktopServices::openUrl(QUrl("https://api.twitter.com/oauth/authorize?" + QOAuth::tokenParameterName() + "=" + token));

		bool pinValid = false;
		QString pin = QInputDialog::getText(this, "Pin", "Enter pin",QLineEdit::Normal, QString(), &pinValid);

		if(pinValid)
		{
			QOAuth::ParamMap args2;
				args2.insert("oauth_verifier", pin.toAscii());

			reply = oauthInterface->accessToken("https://api.twitter.com/oauth/access_token", QOAuth::GET, token, tokenSecret, QOAuth::HMAC_SHA1, args2);

			if(oauthInterface->error() == QOAuth::NoError)
			{
				emit gotAccess(reply.value("screen_name"),
							   reply.value(QOAuth::tokenParameterName()),
							   reply.value(QOAuth::tokenSecretParameterName()));
			}
			else
			{
				qDebug() << "[QOAuth::Interface ERROR]" << oauthInterface->error();
			}
		}
	}
	else
	{
		qDebug() << "[QOAuth::Interface ERROR]" << oauthInterface->error();
	}
}

/*
 * ################################################################################################
 * #                                       FriendsFetcher                                       #
 * ################################################################################################
 */

FriendsFetcher::FriendsFetcher(QObject *parent) : QObject(parent)
{
	_oauthInterface = new QOAuth::Interface(this);
	_manager = new QNetworkAccessManager(this);
}

FriendsFetcher::~FriendsFetcher()
{
	delete reply;
	delete _manager;
	delete _oauthInterface;
}

void FriendsFetcher::run(int id, QString cursor, QString token, QString tokenSecret, int tokenId)
{
	this->id = id;
	this->cursor = cursor;
	this->tokenId = tokenId;

	qDebug() << "Getting friends for id" << id << "with cursor" << cursor;


	_oauthInterface->setConsumerKey(CONSUMER_KEY);
	_oauthInterface->setConsumerSecret(CONSUMER_SECRET_KEY);

	QNetworkRequest request;

	QOAuth::ParamMap params;
		params.insert("user_id", QString::number(id).toAscii());
		params.insert("cursor", cursor.toAscii());

	QString url("https://api.twitter.com/1/friends/ids.json");
	QByteArray header = _oauthInterface->createParametersString(url,
																QOAuth::GET,
																token.toUtf8(),
																tokenSecret.toUtf8(),
																QOAuth::HMAC_SHA1,
																params,
																QOAuth::ParseForHeaderArguments);
	if(!params.isEmpty())
		url.append(_oauthInterface->inlineParameters(params, QOAuth::ParseForInlineQuery));
	request.setUrl(url);
	request.setRawHeader("Authorization", header);

	QEventLoop pause;
	QObject::connect(_manager, SIGNAL(finished(QNetworkReply*)), &pause, SLOT(quit()));
	this->reply = _manager->get(request);
	pause.exec();
}

/*
 * ################################################################################################
 * #                                        FriendRunnable                                        #
 * ################################################################################################
 */

FriendRunnable::FriendRunnable(int id, QString cursor, Setting *settings)
{
	_id = id;
	_cursor = cursor;
	_settings = settings;
}
void FriendRunnable::run()
{
	FriendsFetcher *friendsFetcher = new FriendsFetcher;
	friendsFetcher->run(_id, _cursor, _settings->twitter->id(), _settings->twitter->psswd(), _settings->twitter->index());
	emit gotReply(friendsFetcher);
}

/*
 * ################################################################################################
 * #                                          ApiTwitter                                          #
 * ################################################################################################
 */

ApiTwitter::ApiTwitter(Graph **graph_ptr, Setting *settings)
{
	ApiTwitter(NULL, graph_ptr, settings);
}

ApiTwitter::ApiTwitter(QWidget *parent, Graph **graph_ptr, Setting *settings) : QWidget(parent),
																				_graph_ptr(graph_ptr),
																				_settings(settings)
{
	_threadPool = new QThreadPool;
	_threadPool->setMaxThreadCount(QThread::idealThreadCount());
}

ApiTwitter::~ApiTwitter()
{
	delete _threadPool;
}

void ApiTwitter::run()
{
	if(_threadPool->activeThreadCount() == 0)
	{
		_settings->logWrite("Updating followers");
		for(NodeSet::Iterator it = (*_graph_ptr)->nodeSet.begin(); it != (*_graph_ptr)->nodeSet.end(); it++)
			updateFollowers((*it)->id, "-1");
	}
}

void ApiTwitter::updateFollowers(int id, QString cursor)
{
	FriendRunnable *friendThread = new FriendRunnable(id, cursor, _settings);

	QObject::connect(friendThread, SIGNAL(gotReply(FriendsFetcher*)), this, SLOT(friendsHandler(FriendsFetcher*)));
	_threadPool->start(friendThread);
}

void ApiTwitter::friendsHandler(FriendsFetcher* friendsFetcher)
{
	//_settings->logWrite("Got friends for id " + QString::number(friendsFetcher->id) + " with cursor " + friendsFetcher->cursor);

	QJson::Parser parser;
	bool parsed;
	QVariantMap response = parser.parse(friendsFetcher->reply, &parsed).toMap();
	if(parsed)
	{
		if(response["error"].toString() == "")
		{
			foreach (QVariant tweet, response["ids"].toList())
			{
				if((*_graph_ptr) != 0)
					(*_graph_ptr)->addEdge(edgeFromPair(friendsFetcher->id, tweet.toInt(), QDateTime::currentDateTimeUtc()));
			}
			if(response["next_cursor_str"].toString() != "0")
			{
				updateFollowers(friendsFetcher->id, response["next_cursor_str"].toString());
			}
		}
		else
		{
			if(friendsFetcher->tokenId == _settings->twitter->index())
			{
				_settings->logWrite("Twitter account " + QString::number(friendsFetcher->tokenId) + " depleted");
				_settings->twitter->next();
			}
			if(_settings->running)
				updateFollowers(friendsFetcher->id, friendsFetcher->cursor);
		}
	}
	else
	{
		qDebug() << "Error parsing twitter response:";
		qDebug() << friendsFetcher->reply->readAll();
		qApp->exit(1);
	}
	friendsFetcher->deleteLater();
}


